//
// Caching.js
// FernsehWidget
// Vom Pascal Pfiffner und Sebastian Krau
//


var cacheBase;
var bgCacheReturnFunction;
var bgCacheShows;
var bgCacheRequest;
var bgCachedShows;
var bgCacheListingDay;
var bgCacheChannel;
var bgCacheChannelArray;
var cacheFurtherURL;
var parseErrorOccurred;
var networkErrorOccurred = false;
var weMayCache = true;
var didActuallyLoad = false;
var wasFirstHalfOfDay;
var cachingDate;



function localUrl(channel, listingday)
{
	var date = new Date();
	listingday = parseInt(listingday);
	
	date.setTime(date.getTime() + ((8 == listingday ? -1 : listingday) * 86400000));
	localurl = cacheBase + (date.getMonth() + 1) + "-" + date.getDate() + "-" + channel;
		
	return localurl;
}

function clearCache()
{
	// Clearing the cache is a matter of fractions of a second, however, it is nice to give some feedback, so fake it
	document.getElementById("clear_cache_button").style.opacity = 0.4;
	setTimeout('document.getElementById("clear_cache_button").style.opacity = 1;', 500);
	FernsehPlugin.emptyCache();
}

//
// returns whether cached data is available for all channels
//
function cachedDataAvailable(channels, listingday)
{
	var channel;
	var y;
	for (y = 0; y < channels.length; y++)
	{
		channel = channels[y];
		var channelshows = FernsehPlugin.cat(localUrl(channel, listingday));
		if ((undefined == channelshows) || ("" == channelshows))
		{
			debug(channel + " " + listingday + " not available");
			return false;
		}
		debug(channel + " " + listingday + " available");
	}
	return true;
}

//
// Asynchronous caching
// Note: takes an array of channels
//
function backgroundCachedData(channels, listingday, returnFunc)
{
	var everythingThere = true;
	var channel;
	if (listingday >= 0)
	{
		// check whether a network error occurred. if yes, the returnfunction deals with it
		if (networkErrorOccurred)
		{
			returnFunc();
			bgCacheChannelArray = undefined;
			stopCacheIndicator();
			return;
		}
	
		bgCacheReturnFunction = returnFunc;
		parseErrorOccurred = false;
		
		var bgCacheShows = new Object();
		
		var everythingThere = true;
		
		var i;
		for (i = 0; i < channels.length; i++)
		{
			channel = channels[i];
			if ((undefined == channel) || (null == channel)) continue;
			var channelshows = loadShows(localUrl(channel, listingday));
			if (undefined == channelshows)
			{
				everythingThere = false;
				break;
			}
			bgCacheShows[channel] = channelshows;
		}
		
		bgCacheChannelArray = channels;
	} else everythingThere = true;
	
	if (everythingThere)
	{	
		returnFunc();
		stopCacheIndicator();
		// only update the badge if caching really happened.
		//if (null != bgCacheChannel) updateFilterBadge();
		// moved to forwardCache() in main.js
		// less annoying if only done once after all caching has been done
		return;
	} else {
		// else cache them and then return them
		startCacheIndicator();
		setCacheIndicatorText("Lade " + channelNameFromInternalName(parseChannelName(channel)));
		loadInBackground(channel, listingday);
	}
}

function dataUrl(listingday, channel)
{
	var date = new Date();
	listingday = parseInt(listingday);
	
	listingday = (8 == listingday ? -1 : listingday);
	date.setTime(date.getTime() + (listingday * 86400000));
		
	localurl = cacheBase + (date.getMonth() + 1) + "-" + date.getDate() + "-" + channel;
	var day = date.getDate();
	cachingDate = date;
	if (day < 10) day = "0" + day;
	var month = date.getMonth() + 1;
	if (month < 10) month = "0" + month;
	var year = date.getFullYear();
	var fulldate = day + "." + month + "." + year;
	
	var requesturl = "http://www.tvtoday.de/program2007?format=genre&date=" + fulldate + "&slotIndex=all&genre=&channel=" + channel;
	
	debug("requesturl: " + requesturl + " listingday " + listingday);
	return requesturl;
}

//
// Starts asynchronous loading
//
function loadInBackground(channel, listingday)
{
	if (!weMayCache) return;
	bgCacheChannel = channel;
	bgCacheListingDay = parseInt(listingday);
	networkErrorOccurred = false;
	
	bgCachedShows = new Object();
	var requesturl = dataUrl(bgCacheListingDay-1, channel);
	wasFirstHalfOfDay = true;

	bgCacheRequest = new XMLHttpRequest;
	bgCacheRequest.onreadystatechange = bgProcessReqChange;
	bgCacheRequest.open("GET", requesturl, true);
	bgCacheRequest.setRequestHeader("Cache-Control", "no-cache");
	bgCacheRequest.setRequestHeader('User-Agent','Fernsehen Widget 1.8 widgetschmie.de/widgets/Fernsehen');
	bgCacheRequest.send(null);
}

function bgProcessReqChange()
{
	if (parseErrorOccurred) return;
	if (bgCacheRequest.readyState != 4) return;

	if ((!((bgCacheRequest.status >= 200) && (bgCacheRequest.status < 400))) || (bgCacheRequest.status == undefined))
	{
		networkErrorOccurred = true;
		bgCacheChannel = null;
		backgroundCachedData(bgCacheChannelArray, bgCacheListingDay, bgCacheReturnFunction);
		stopCacheIndicator();
		// showError();
		// deal with error here
		return;
	}
	networkErrorOccurred = false;
	var newShows = parseShows(bgCacheRequest.responseText, bgCacheListingDay, !wasFirstHalfOfDay);

	if (undefined == newShows)
	{
		networkErrorOccurred = true;
		alert("parseShows() was unsuccessful, abort caching here");
		bgCacheChannel = null;
		backgroundCachedData(bgCacheChannelArray, bgCacheListingDay, bgCacheReturnFunction);
		return;
	}
	
	for (var time in newShows)
	{
		bgCachedShows[time] = newShows[time];			
	}
	
	if (wasFirstHalfOfDay)
	{
		if (!weMayCache) return;
		var requesturl = dataUrl(bgCacheListingDay, bgCacheChannel);
		wasFirstHalfOfDay = false;
	
		bgCacheRequest = new XMLHttpRequest;
		bgCacheRequest.onreadystatechange = bgProcessReqChange;
		bgCacheRequest.open("GET", requesturl, true);
		bgCacheRequest.setRequestHeader("Cache-Control", "no-cache");
		bgCacheRequest.setRequestHeader('User-Agent','Fernsehen Widget 1.8 widgetschmie.de/widgets/Fernsehen');
		bgCacheRequest.send(null);
	} else {
		// We have all the data for a complete day
		debug("bgProcessReqChange complete");
		checkAlarmFilter(bgCachedShows, bgCacheChannel);
		saveShows(bgCachedShows, localUrl(bgCacheChannel, bgCacheListingDay));
		
		// call this function again, it knows what other channels have to be cached and calls the callback if all data has been loaded
		bgCacheChannel = null;
		//updateFilterBadge();
		backgroundCachedData(bgCacheChannelArray, bgCacheListingDay, bgCacheReturnFunction);
	}
}

//
// Synchronous caching
// Note: takes the name of one channel
//
function cachedData(channel, listingday)
{
	if ((undefined == channel) || (null == channel))
	{
		alert("undefined channel!");
		return;
	}
	var shows = loadShows(localUrl(channel, listingday));
	
	if (shows != undefined)
	{	
		return shows;
	} else {
		// else cache them and then return them
		startCacheIndicator();
		setCacheIndicatorText("Lade " + parseChannelName(channel));
		
		shows = cacheData(channel, listingday);
		stopCacheIndicator();
		if (undefined == shows)
		{
			// showError();
			return;
		} else {
			updateFilterBadge();
			return shows;
		}
	}
}


//
// Synchronous caching
//
function cacheData(channel, listingday)
{
	var shows = new Object();
	
	if (undefined == listingday) return;
	if (undefined == channel) return;
	if (null == channel) return;
	listingday = parseInt(listingday);
	if (listingday == 8) listingday = -1;
	var request = new XMLHttpRequest;
	bgCacheListingDay = listingday; // parseShows relies on this
	var requesturl = dataUrl(listingday-1, channel);
	if (listingday >= 0)
	{
		request.open("GET", requesturl, false);
		request.setRequestHeader("Cache-Control", "no-cache");
		request.setRequestHeader('User-Agent','Fernsehen Widget 1.8 widgetschmie.de/widgets/Fernsehen');
		request.send(null);
		if (!((request.readyState == 4) && (request.status >= 200) && (request.status < 400))) return;
		var newShows = parseShows(request.responseText, listingday, false);
		
		if (undefined == newShows)
		{
			alert("parseShows() was unsuccessful, abort caching here");
			return;
		}
		
		for (var time in newShows)
		{
			shows[time] = newShows[time];			
		}
	}
	request = new XMLHttpRequest;
	requesturl = dataUrl(listingday, channel);
	request.open("GET", requesturl, false);
	request.setRequestHeader("Cache-Control", "no-cache");
	request.setRequestHeader('User-Agent','Fernsehen Widget 1.8 widgetschmie.de/widgets/Fernsehen');
	request.send(null);
	if (!((request.readyState == 4) && (request.status >= 200) && (request.status < 400))) return;
	var newShows = parseShows(request.responseText, listingday, true);
	
	if (undefined == newShows)
	{
		alert("parseShows() was unsuccessful, abort caching here");
		return;
	}
	
	for (var time in newShows)
	{
		shows[time] = newShows[time];			
	}
	
	checkAlarmFilter(shows, channel);
	saveShows(shows, localUrl(channel, listingday));
	return shows;
}

function saveShows(shows, filename)
{
	var saveShows = new Object();
	for (var time in shows)
	{
		var variablesavetime = "_" + time.replace(/\./g, "_");
		saveShows[variablesavetime] = shows[time];
	}
	var serialized = Serialize(saveShows);
	var encoded = serialized.replace(/"/g, "&quot;");			// " (editor)
	FernsehPlugin.saveCachedData(encoded, filename);
	debug("saving to "+filename);
}

//
// Load from disk
//
function loadShows(filename)
{
	// The plugin handles encoding and spaces in filenames better
	var file = FernsehPlugin.cat(filename);
	
	if ((undefined == file) || ("" == file)) {
		return;
	}
	
	var rawobject = file.replace(/&quot;/g, "\"");
	var deserialized = Deserialize(rawobject);
	var newObj = new Object();
	var foo;
	for(var key in deserialized) {
		foo = key.replace(/_/g, ".");
		newObj[foo.substr(1,5)] = deserialized[key];
	}
	
	return newObj;
}

function parseShows(rawsource, listingday, firstHalfOfDay)
{
	// Das alles zu einer Zeile machen 
	var dateFirstShow;
	lines = rawsource.split("\n");
	rsource = lines.join("");
	var newShows = new Object();
	var doBreak = false;
	var inSecondHalf = false;
	var lastEncounteredHour = 0;
	
	// abort if we got no valid data
	var checkresult = rawsource.match(/In diesem Zeitraum wurden zu Ihrer Auswahl leider keine Sendungen gefunden/g);
	if ((null != checkresult) && (undefined != checkresult) && (checkresult.length == 6)) {
		alert("Ungueltige Suchanfrage an TVToday. Da muss ich passen, sorry.");
		return;
	}
	
	
	/***********************
	 * parsing starts here *
	 ***********************/
	
	try {
		
		// Der entscheidende Teil fr das Fernsehprogramm...
		rmatch = /<div id="program_complete">(.*?)tvtoday.de\/img\/program\/verlauf_fuss\.jpg(.*)$/;
	
		rmatch.exec(rsource);
		var listing = RegExp.$1;
		/*if (listing != undefined) debug("habe listing");
		else debug("kein listing");*/
		
		shows = new Array();
		showsregexp = /<td width="45" style="border-right: 2px solid white; border-bottom: 2px solid white; border-left: 2px solid white;">(.*?)<\/a><br\/>\s*<\/td>\s*<\/tr>/g;
		
		rawshows = listing.match(showsregexp);
		
		debug("num shows " + rawshows.length);
		
		var startstamp;
		var searchdate;
		var newShows = new Object();
		
		if (8 == listingday) listingday = -1;
		
		// Loop through the single shows
		var lastStartTime;
		
		for (i = 0; i < rawshows.length; i++)
		{
			//debug(rawshows[i]);
			//debug(rawshows[i]);
			
			iconurlregexp = /<img src="([^"]*)" id="channel_logo_[^"]*" alt="[^"]*" class="channel_logo" \/>/;
			iconurlraw = rawshows[i].match(iconurlregexp);
			iconurl = iconurlraw[1];
			
			debug(iconurl);
			
			/*******************************
			 * Start the Date/Time parsing *
			 *******************************/
			
			// RegExp to get the time from tvtoday
			starttimeregexp = /<span class="tv-sendung-uhrzeit">([0-9.]*)<\/span>/;
			starttimeraw = rawshows[i].match(starttimeregexp);
			starttime = starttimeraw[1];
			
			debug(starttime);
			
			// End Date/Time
			
			shownameregexp = /<td style="border-bottom: 2px solid white;"><a href="javascript: detailPopup\('\d+'\);" class="tv-sendung-titel">([^<]*)<\/a>/;
			shownameraw = rawshows[i].match(shownameregexp);
			showname = shownameraw[1];
			
			debug(showname);

			showurlregexp = /<td style="border-bottom: 2px solid white;"><a href="javascript: detailPopup\('(\d+)'\);" class="tv-sendung-titel">/; 
			showurlraw = rawshows[i].match(showurlregexp);
			var showurl = null;
			if (showurlraw != null) showurl = "http://www.tvtoday.de/program2007?format=detail&sid=" + showurlraw[1];
			debug(showurl);
			
			showchannelregexp = /<span class="tv-sendung-info">\(<a href="\?format=channel&channelId=[^"]*"><b>([^<]*)<\/b>/;
			showchannel = rawshows[i].match(showchannelregexp);
			channel = showchannel[1];
			
			debug(channel);

			showdescregexp = /<span class="tv-sendung-info">\(<a href="\?format=channel&channelId=[^"]*"><b>[^<]*<\/b><\/a>\)\s*(-?\d*?\/)?(-?\d+ Min.\s*?bis [0-9.]*) Showview [0-9\-]*<br>\s*(.*?)\s*<\/span>/;
			showdescraw = rawshows[i].match(showdescregexp);
			
			showlength = "";
			showlength = (undefined != showdescraw[2] ? showdescraw[2] : "30 Min. (geraten)");
			if (parseInt(showlength) < 0) // oh, this has happened before...
			{
				alert("Weird show length. Assuming 30 minutes.");
				showlength = "30 Min. (geraten)";
			}
			debug(showlength);
			showdesc = "";
			
			if ((showdescraw.length > 3) && (undefined != showdescraw[3])) showdesc += showdescraw[3];
			debug(showdesc);
			
			var thisHour = Math.abs(starttime.split(".")[0]);
			if (thisHour < lastEncounteredHour)
			{
				debug("midnight reached");
				inSecondHalf = true;
				if (firstHalfOfDay)
				{
					doBreak = true;
					break;
				}
			}
			lastEncounteredHour = thisHour;
			
			if ((firstHalfOfDay && !inSecondHalf) || (!firstHalfOfDay && inSecondHalf))
			{
				var thisshow = new Object();
				
				thisshow["title"] = showname;
				thisshow["length"] = showlength;
				if ((undefined != showdesc) && ("" != showdesc)) thisshow["description"] = showdesc;
				if ((undefined != showurl) && ("" != showurl)) thisshow["url"] = showurl;
				if ((undefined != iconurl) && ("" != iconurl)) thisshow["icon"] = iconurl;
				
				// get a real timestamp for the startdate
				thisshowhour = Math.abs((starttime.split("."))[0]);
				
				var date = new Date();
				date.setTime(date.getTime() + (listingday * 86400000));
				date.setHours(thisshowhour);
				date.setMinutes(Math.abs((starttime.split("."))[1]));
				date.setSeconds(0);
				thisshow["timestamp"] = date;
				
				// calculate an endstamp
				thisshow["endtime"] = new Date();
				if("" != showlength) {
					var durparts = showlength.split(" ");
					thisshow["durationInMin"] = Math.abs(durparts[0]);
					thisshow["endtime"].setTime(date.getTime() + (durparts[0] * 60 * 1000));
				}
				else {
					alert("Got no duration for " + thisshow["title"] + " on " + channel + ", assuming 30 Minutes");
					thisshow["durationInMin"] = 30;
					thisshow["endtime"].setTime(Math.abs(date.getTime() + 1800000));		// to be on the safe side and at least have an endstamp (30min after start)
				}
				debug(showname);
				debug(starttime);
							
				if (!doBreak) newShows[starttime] = thisshow;
			}
		}	
	}
	
	// error cought
	catch(exc)
	{	
		parseErrorOccurred = true;
		stopCacheIndicator();
		
		document.getElementById('content').style.top = '0px';
	 	document.getElementById("previousShows").style.opacity = 0.4;
		document.getElementById("nextShows").style.opacity = 0.4;
		
		alert("Fehler beim Parsen des Programms (Tag " + listingday + ": " + cachingDate.toLocaleString() + ", Source " + rawsource.length + " Zeichen): " + exc);
		showDialog("Beim Analysieren der Daten von TVToday ist ein Fehler aufgetreten.<br /><br />Dies kann unterschiedliche Ursachen haben, die wahrscheinlichste ist da&szlig; TVToday das Format seiner Seiten ver&auml;ndert hat.<br />In diesem Fall k&ouml;nnen Sie nur auf ein Update seitens der Entwickler warten.", "Erneut versuchen", updateTVListing, "Webseite &ouml;ffnen", gotoWebsite);
		return newShows;
	}
	
	debug("num shows parsed "+(keysFromHash(newShows)).length);

	return newShows;
}
